#!/user/bin/env python

import sys

try:
    import django
except ImportError:
    print("Error: missing test dependency:")
    print("  django library is needed to run test suite")
    print("  you can install it with 'pip install django'")
    print("  or use tox to automatically handle test dependencies")
    sys.exit(1)

from django.conf import settings


def main():
    settings.configure(
        INSTALLED_APPS=[
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.sitemaps',
            'qanda',
            ],
        # Django replaces this, but it still wants it. *shrugs*
        DATABASE_ENGINE='django.db.backends.sqlite3',
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': ':memory:',
                }
            },
        MEDIA_ROOT='/tmp/qanda_test_media/',
        MEDIA_PATH='/media/',
        ROOT_URLCONF='qanda.tests.urls',
        DEBUG=True,
        TEMPLATE_DEBUG=True,
        MIDDLEWARE_CLASSES=[
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.middleware.common.CommonMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
            'django.middleware.clickjacking.XFrameOptionsMiddleware',
            ]
        )

    if django.VERSION[:2] >= (1, 7):
        django.setup()

    apps = ['qanda']
    # if django.VERSION[:2] >= (1, 6):
    #    apps.append('qanda.tests')

    from django.core.management import call_command
    from django.test.utils import get_runner

    try:
        from django.contrib.auth import get_user_model
    except ImportError:
        USERNAME_FIELD = 'username'
    else:
        USERNAME_FIELD = get_user_model().USERNAME_FIELD

    DjangoTestRunner = get_runner(settings)

    class TestRunner(DjangoTestRunner):
        def setup_databases(self, *args, **kwargs):
            result = super(TestRunner, self).setup_databases(*args, **kwargs)
            kwargs = {
                'interactive': False,
                'email': 'admin@doesnotexit.com',
                USERNAME_FIELD: 'admin',
                }
            call_command("createsuperuser", **kwargs)
            return result

    failures = TestRunner(verbosity=2, interactive=True).run_tests(apps)
    sys.exit(failures)



from django.core.urlresolvers import reverse
from django.db import models
from django.utils import timezone
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext as _


class AutoTimestampBase(models.Model):
    """Abstract base class to automatically set timestamps.

    Adds the fields 'created' and 'modified' to child classes.
    """

    created = models.DateTimeField(_('created'), blank=True, null=True)
    modified = models.DateTimeField(_('modified'), blank=True, null=True)

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        """
        Set the modified and created, if required, timestamps.
        """
        self.modified = timezone.now()
        if not self.created:
            self.created = self.modified
        super(AutoTimestampBase, self).save(*args, **kwargs)


class TopicManager(models.Manager):

    def for_staff(self):
        """
        Return all topics whose questions are published.
        """
        qs = super(TopicManager, self).get_queryset()
        return qs.distinct().filter(questions__is_published=True)

    def for_members(self):
        """
        Return all topics whose questions are published and have public or
        members access level.
        """
        return self.distinct().filter(
            questions__is_published=True,
            questions__access__in=[Question.PUBLIC, Question.MEMBERS]
            )

    def for_anonymous(self):
        """
        Return all topics whose questions are published and
        have public access level.
        """
        return self.distinct().filter(
            questions__is_published=True,
            questions__access=Question.PUBLIC
            )


@python_2_unicode_compatible
class Topic(models.Model):
    """
    Generic Topics for FAQ questions.
    """
    title = models.CharField(_('title'), max_length=150, unique=True)
    slug = models.SlugField(_('slug'), max_length=150, unique=True)
    description = models.TextField(_('description'), blank=True)
    order = models.IntegerField(_('sort order'), default=0)

    # Managers
    objects = TopicManager()

    class Meta:
        ordering = ['order', 'title', ]
        verbose_name = _('topic')
        verbose_name_plural = _('topics')

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse(
            'qanda.views.topicdetail',
            kwargs={'topic_slug': self.slug}
            )


class PublishedQuestionManager(models.Manager):

    use_for_related_fields = True

    def get_queryset(self):
        """
        Return all questions which are published.
        """
        qs = super(PublishedQuestionManager, self).get_queryset()
        return qs.filter(is_published=True)

    def for_members(self):
        """
        Return all questions which have public or members access.
        """
        return self.filter(access__in=[self.model.PUBLIC, self.model.MEMBERS])

    def for_anonymous(self):
        """
        Return all questions which have public access.
        """
        return self.filter(access=self.model.PUBLIC)


@python_2_unicode_compatible
class Question(AutoTimestampBase):
    """
    A frequently asked question.
    """
    text = models.TextField(_('question'))
    slug = models.SlugField(_('slug'), max_length=100, unique=True)
    answer = models.TextField(_('answer'), blank=True)
    topic = models.ForeignKey(
        Topic,
        verbose_name=_('topic'),
        related_name='questions'
        )

    is_published = models.BooleanField(_('is published?'), default=False)
    PUBLIC = 0
    MEMBERS = 1
    STAFF = 2
    ACCESS_CHOICES = (
        (PUBLIC, _('Public')),
        (MEMBERS, _('Members')),
        (STAFF, _('Staff')),
        )
    access = models.PositiveSmallIntegerField(
        _('access'),
        choices=ACCESS_CHOICES,
        default=PUBLIC,
        )

    order = models.IntegerField(_('sort order'), default=0)

    # Managers
    objects = models.Manager()
    published = PublishedQuestionManager()

    class Meta:
        ordering = ['order', 'created', ]
        verbose_name = _('question')
        verbose_name_plural = _('questions')

    def __str__(self):
        return self.text

    def get_absolute_url(self):
        return reverse(
            'qanda.views.questiondetail',
            kwargs={
                'topic_slug': self.topic.slug,
                'question_slug': self.slug
                }
            )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(null=True, verbose_name='created', blank=True)),
                ('modified', models.DateTimeField(null=True, verbose_name='modified', blank=True)),
                ('text', models.TextField(verbose_name='question')),
                ('slug', models.SlugField(unique=True, max_length=100, verbose_name='slug')),
                ('answer', models.TextField(verbose_name='answer', blank=True)),
                ('is_published', models.BooleanField(default=False, verbose_name='is published?')),
                ('access', models.PositiveSmallIntegerField(default=0, verbose_name='access', choices=[(0, 'Public'), (1, 'Members'), (2, 'Staff')])),
                ('order', models.IntegerField(default=0, verbose_name='sort order')),
            ],
            options={
                'ordering': ['order', 'created'],
                'verbose_name': 'question',
                'verbose_name_plural': 'questions',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Topic',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(unique=True, max_length=150, verbose_name='title')),
                ('slug', models.SlugField(unique=True, max_length=150, verbose_name='slug')),
                ('description', models.TextField(verbose_name='description', blank=True)),
                ('order', models.IntegerField(default=0, verbose_name='sort order')),
            ],
            options={
                'ordering': ['order', 'title'],
                'verbose_name': 'topic',
                'verbose_name_plural': 'topics',
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='question',
            name='topic',
            field=models.ForeignKey(related_name=b'questions', verbose_name='topic', to='qanda.Topic'),
            preserve_default=True,
        ),
    ]



from django.contrib.sitemaps import Sitemap
from .models import Question


class QuestionSitemap(Sitemap):
    changefreq = 'weekly'
    priority = 0.5

    def items(self):
        return Question.published.for_anonymous()

    def lastmod(self, obj):
        return obj.modified



PUBLISHED_PUBLIC_SLUGS = [
    'public-question-1',
    'public-question-2',
    'public-question-4',
    ]
PUBLISHED_MEMBER_SLUGS = [
    'members-question-1',
    'members-question-2',
    'members-question-4',
    ]
PUBLISHED_STAFF_SLUGS = [
    'staff-question-1',
    'staff-question-2',
    'staff-question-4',
    ]
TOTAL_QUESTIONS_COUNT = 12

PUBLIC_TOPIC_SLUGS = [
    'public-topic',
    'mixed-topic',
    ]
MEMBERS_TOPIC_SLUGS = [
    'public-topic',
    'members-topic',
    'staff-and-members-topic',
    'mixed-topic',
    ]
STAFF_TOPIC_SLUGS = [
    'public-topic',
    'members-topic',
    'staff-topic',
    'staff-and-members-topic',
    'mixed-topic',
    ]
TOTAL_TOPIC_COUNT = 5



from django.test import TestCase
from qanda.models import Question, Topic
from .common_data import (
    PUBLISHED_PUBLIC_SLUGS,
    PUBLISHED_MEMBER_SLUGS,
    PUBLISHED_STAFF_SLUGS,
    TOTAL_QUESTIONS_COUNT,
    PUBLIC_TOPIC_SLUGS,
    MEMBERS_TOPIC_SLUGS,
    STAFF_TOPIC_SLUGS,
    TOTAL_TOPIC_COUNT
    )


class PublishedQuestionMangerTestCase(TestCase):

    fixtures = ['qanda_test.json']

    def test_total_question_count(self):
        self.assertEqual(Question.objects.count(), TOTAL_QUESTIONS_COUNT)

    def test_questions_visible_to_anonymous(self):
        qs = Question.published.for_anonymous()
        expected_slugs = PUBLISHED_PUBLIC_SLUGS
        got_slugs = qs.values_list('slug', flat=True)
        try:
            self.assertItemsEqual(expected_slugs, got_slugs)
        except AttributeError:
            self.assertEqual(sorted(expected_slugs), sorted(got_slugs))

    def test_questions_visible_to_members(self):
        qs = Question.published.for_members()
        expected_slugs = PUBLISHED_PUBLIC_SLUGS + PUBLISHED_MEMBER_SLUGS
        got_slugs = qs.values_list('slug', flat=True)
        try:
            self.assertItemsEqual(expected_slugs, got_slugs)
        except AttributeError:
            self.assertEqual(sorted(expected_slugs), sorted(got_slugs))

    def test_questions_visible_to_staff(self):
        qs = Question.published.all()
        expected_slugs = (
            PUBLISHED_PUBLIC_SLUGS +
            PUBLISHED_MEMBER_SLUGS +
            PUBLISHED_STAFF_SLUGS
            )
        got_slugs = qs.values_list('slug', flat=True)
        try:
            self.assertItemsEqual(expected_slugs, got_slugs)
        except AttributeError:
            self.assertEqual(sorted(expected_slugs), sorted(got_slugs))


class TopicManagerTestCase(TestCase):

    fixtures = ['qanda_test.json']

    def test_total_topic_count(self):
        self.assertEqual(Topic.objects.count(), TOTAL_TOPIC_COUNT)

    def test_topics_visible_to_anonymous(self):
        qs = Topic.objects.for_anonymous()
        expected_slugs = PUBLIC_TOPIC_SLUGS
        got_slugs = qs.values_list('slug', flat=True)
        try:
            self.assertItemsEqual(expected_slugs, got_slugs)
        except AttributeError:
            self.assertEqual(sorted(expected_slugs), sorted(got_slugs))

    def test_topics_visible_to_members(self):
        qs = Topic.objects.for_members()
        expected_slugs = MEMBERS_TOPIC_SLUGS
        got_slugs = qs.values_list('slug', flat=True)
        try:
            self.assertItemsEqual(expected_slugs, got_slugs)
        except AttributeError:
            self.assertEqual(sorted(expected_slugs), sorted(got_slugs))

    def test_topics_visible_to_staff(self):
        qs = Topic.objects.for_staff()
        expected_slugs = STAFF_TOPIC_SLUGS
        got_slugs = qs.values_list('slug', flat=True)
        try:
            self.assertItemsEqual(expected_slugs, got_slugs)
        except AttributeError:
            self.assertEqual(sorted(expected_slugs), sorted(got_slugs))



from django.contrib import admin
from .models import Topic, Question


def action_message(model, count, actioned_phrase):
    singular = model._meta.verbose_name.lower()
    plural = model._meta.verbose_name_plural.lower()
    if count == 1:
        message_bit = '1 {} was'.format(singular)
    else:
        message_bit = '{} {} were'.format(count, plural)
    return '{} successfully {}.'.format(message_bit, actioned_phrase)


def make_published(modeladmin, request, queryset):
    actioned_phrase = 'published'
    row_count = queryset.update(is_published=True)
    modeladmin.message_user(
        request,
        action_message(queryset.model, row_count, actioned_phrase)
        )
make_published.short_description = 'Publish selected questions'


def make_unpublished(modeladmin, request, queryset):
    actioned_phrase = 'unpublished'
    row_count = queryset.update(is_published=False)
    modeladmin.message_user(
        request,
        action_message(queryset.model, row_count, actioned_phrase)
        )
make_unpublished.short_description = 'Unpublish selected questions'


def make_staff_access(modeladmin, request, queryset):
    actioned_phrase = 'given staff access'
    row_count = queryset.update(access=Question.STAFF)
    modeladmin.message_user(
        request,
        action_message(queryset.model, row_count, actioned_phrase)
        )
make_staff_access.short_description = 'Give staff access to selected questions'


def make_member_access(modeladmin, request, queryset):
    actioned_phrase = 'given member access'
    row_count = queryset.update(access=Question.MEMBERS)
    modeladmin.message_user(
        request,
        action_message(queryset.model, row_count, actioned_phrase)
        )
make_member_access.short_description = (
    'Give member access to selected questions'
    )


def make_public_access(modeladmin, request, queryset):
    actioned_phrase = 'given public access'
    row_count = queryset.update(access=Question.PUBLIC)
    modeladmin.message_user(
        request,
        action_message(queryset.model, row_count, actioned_phrase)
        )
make_public_access.short_description = (
    'Give public access to selected questions'
    )


class TopicAdmin(admin.ModelAdmin):
    list_display = ('title',)
    prepopulated_fields = {'slug': ('title',)}


class QuestionAdmin(admin.ModelAdmin):
    list_display = ('text', 'topic', 'access', 'is_published', 'modified')
    list_filter = ('is_published', 'access', 'topic')
    readonly_fields = ('created', 'modified')
    search_fields = ('text', 'answer')
    prepopulated_fields = {'slug': ('text',)}
    actions = [
        make_published,
        make_unpublished,
        make_staff_access,
        make_member_access,
        make_public_access
        ]

admin.site.register(Topic, TopicAdmin)
admin.site.register(Question, QuestionAdmin)



"""
Mixins for qanda.views
"""

# Django
from django.shortcuts import get_object_or_404
from django.utils.functional import cached_property

# Local
from .models import Question, Topic


class QuestionQuerysetMixin(object):
    question_queryset = Question.published

    def get_question_queryset(self):
        queryset = self.question_queryset

        # Check for access level
        if self.request.user.is_staff:
            pass
        elif self.request.user.is_authenticated():
            queryset = queryset.for_members()
        else:
            queryset = queryset.for_anonymous()
        return queryset


class TopicQuestionsMixin(QuestionQuerysetMixin):
    topic_queryset = Topic.objects
    context_object_list_name = None

    @cached_property
    def queryset(self):
        """
        Override the queryset property filtering by both questions and topic.
        """
        return self.get_question_queryset().filter(topic=self.topic)

    @cached_property
    def topic(self):
        return get_object_or_404(
            self.topic_queryset,
            slug__exact=self.kwargs['topic_slug']
            )

    def get_context_object_list_name(self, object_list):
        """
        Get the name of the object_list to be used in the context.
        """
        if self.context_object_list_name:
            return self.context_object_list_name
        elif hasattr(object_list, 'model'):
            return '%s_list' % object_list.model._meta.model_name
        else:
            return None

    def get_context_data(self, **kwargs):
        """
        Insert the context_object_list into the template context.
        """
        context = super(TopicQuestionsMixin, self).get_context_data(**kwargs)
        list_name = self.get_context_object_list_name(self.queryset)
        if list_name is not None:
            context[list_name] = self.queryset
        return context



# Django
from django.conf import settings
from django.http import Http404
from django.views.generic import ListView, DetailView

# Local
from .models import Topic
from .mixins import QuestionQuerysetMixin, TopicQuestionsMixin

RECENT_UPDATES_COUNT = getattr(settings, 'QANDA_RECENT_UPDATES_COUNT', 7)


class TopicList(QuestionQuerysetMixin, ListView):
    model = Topic
    template_name = 'qanda/topic_list.html'
    allow_empty = True
    questions_context_object_name = 'related_questions'

    def get_queryset(self):
        # Check for access level
        if self.request.user.is_staff:
            return self.model.objects.for_staff()

        if self.request.user.is_authenticated():
            return self.model.objects.for_members()

        return self.model.objects.for_anonymous()

    def get_context_data(self, **kwargs):
        context = super(TopicList, self).get_context_data(**kwargs)
        context_name = self.questions_context_object_name
        questions = self.get_question_queryset()
        if questions:
            questions = questions.order_by('-modified')
            context[context_name] = questions[:RECENT_UPDATES_COUNT]
        return context


class TopicDetail(TopicQuestionsMixin, DetailView):
    template_name = 'qanda/topic_detail.html'
    slug_url_kwarg = 'topic_slug'

    def get_object(self):
        if not self.queryset:
            raise Http404('No questions for that topic')
        return self.topic


class QuestionDetail(TopicQuestionsMixin, DetailView):
    template_name = 'qanda/question_detail.html'
    slug_url_kwarg = 'question_slug'

    def get_object(self):
        qs = self.queryset.filter(topic=self.topic)
        return super(QuestionDetail, self).get_object(qs)



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class QandaAppConfig(AppConfig):

    name = 'qanda'
    verbose_name = _('Frequently Asked Questions')

    def ready(self):
        pass



from django.test import TestCase, Client
from django.core.urlresolvers import reverse
from .common_data import PUBLISHED_PUBLIC_SLUGS
from qanda.sitemaps import QuestionSitemap


class TestSitemapTestCase(TestCase):

    fixtures = ['qanda_test.json']

    def test_question_sitemap(self):
        client = Client()
        response = client.get(reverse(
            'django.contrib.sitemaps.views.sitemap',
            ))
        # We need a 'good' response
        self.assertEqual(response.status_code, 200)
        # ... and it should be XML
        self.assertEqual(response['Content-Type'], 'application/xml')

        # TODO: This should be a smarter check, maybe?
        got_slugs = QuestionSitemap().items().values_list('slug', flat=True)

        try:
            self.assertItemsEqual(PUBLISHED_PUBLIC_SLUGS, got_slugs)
        except AttributeError:
            self.assertEqual(sorted(PUBLISHED_PUBLIC_SLUGS), sorted(got_slugs))



from django.conf.urls import include, url
from django.contrib import admin
from django.conf import settings
from django.contrib.sitemaps.views import sitemap
from qanda.sitemaps import QuestionSitemap

sitemaps = {
    'questions': QuestionSitemap,
    }

urlpatterns = [
    # Examples:
    # url(r'^$', 'example.views.home', name='home'),
    # url(r'^blog/', include('blog.urls')),

    url(r'^admin/', include(admin.site.urls)),

    url(r'^faq/', include('qanda.urls')),

    url(r'^sitemap\.xml$',
        sitemap,
        {'sitemaps': sitemaps},
        name='django.contrib.sitemaps.views.sitemap',
        ),

    # Normally we'd do this if DEBUG only, but this is just an example app.
    url(r'^static/(?P<path>.*)$',
        'django.views.static.serve',
        {'document_root': settings.MEDIA_ROOT}
        ),

    ]



import os
from setuptools import setup
from qanda import __version__

setup(
    name = "django-qanda",
    version = __version__,
    author = "Mike Hurt",
    author_email = "mike@mhtechnical.net",
    description = "A simple FAQ app for Django sites.",
    license = "MIT",
    keywords = "faq question answer django",
    url = "https://bitbucket.org/mhurt/django-qanda",
    packages=['qanda'],
    test_suite = 'runtests.main',
    tests_require=['Django >= 1.6'],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
)



from django.test import TestCase, Client
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse

from .common_data import (
    MEMBERS_TOPIC_SLUGS,
    PUBLIC_TOPIC_SLUGS,
    STAFF_TOPIC_SLUGS,
    )


STAFF_TOPIC_SLUG = 'staff-and-members-topic'
STAFF_QUESTION_SLUGS = ['members-question-2', 'staff-question-2']
STAFF_QUESTION_SLUG = 'staff-question-2'
MEMBERS_QUESTION_SLUGS = ['members-question-2']
MEMBERS_QUESTION_SLUG = 'members-question-2'
MEMBERS_TOPIC_SLUG = 'staff-and-members-topic'
PUBLIC_QUESTION_SLUGS = ['public-question-1', 'public-question-2']
PUBLIC_QUESTION_SLUG = 'public-question-2'
PUBLIC_TOPIC_SLUG = 'public-topic'


class StaffUserTestCase(TestCase):

    fixtures = ['qanda_test.json']

    def setUp(self):
        self.user = User.objects.create_user(
            'staffuser',
            'staff@example.com',
            'password',
            )
        self.user.save()
        self.user.is_staff = True
        self.user.save()
        self.client = Client()
        self.client.login(username='staffuser', password='password')

    def test_staff_topiclist(self):
        response = self.client.get(reverse('qanda.views.topiclist'))
        self.assertEqual(response.context['user'].is_staff, True)

        self.assertEqual(response.status_code, 200)
        got_topics = response.context['object_list']
        got_slugs = [t.slug for t in got_topics]
        try:
            self.assertItemsEqual(STAFF_TOPIC_SLUGS, got_slugs)
        except AttributeError:
            self.assertEqual(sorted(STAFF_TOPIC_SLUGS), sorted(got_slugs))

    def test_staff_topicdetail(self):
        response = self.client.get(reverse(
            'qanda.views.topicdetail',
            kwargs={'topic_slug': 'staff-and-members-topic'}
            ))
        self.assertEqual(response.status_code, 200)
        got_questions = response.context['question_list']
        got_slugs = [q.slug for q in got_questions]
        try:
            self.assertItemsEqual(STAFF_QUESTION_SLUGS, got_slugs)
        except AttributeError:
            self.assertEqual(sorted(STAFF_QUESTION_SLUGS), sorted(got_slugs))

    def test_staff_questiondetail(self):
        response = self.client.get(reverse(
            'qanda.views.questiondetail',
            kwargs={
                'topic_slug': STAFF_TOPIC_SLUG,
                'question_slug': STAFF_QUESTION_SLUG
                }
            ))
        self.assertEqual(response.status_code, 200)
        obj = response.context['object']
        self.assertEqual(obj.slug, STAFF_QUESTION_SLUG)
        self.assertEqual(obj.topic.slug, STAFF_TOPIC_SLUG)


class MemberUserTestCase(TestCase):

    fixtures = ['qanda_test.json']

    def setUp(self):
        self.user = User.objects.create_user(
            'memberuser',
            'member@example.com',
            'password',
            )
        self.user.save()
        self.client = Client()
        self.client.login(username='memberuser', password='password')

    def test_member_topiclist(self):
        response = self.client.get(reverse('qanda.views.topiclist'))
        self.assertFalse(response.context['user'].is_staff)
        self.assertTrue(response.context['user'].is_authenticated())

        self.assertEqual(response.status_code, 200)
        got_topics = response.context['object_list']
        got_slugs = [t.slug for t in got_topics]
        try:
            self.assertItemsEqual(MEMBERS_TOPIC_SLUGS, got_slugs)
        except AttributeError:
            self.assertEqual(sorted(MEMBERS_TOPIC_SLUGS), sorted(got_slugs))

    def test_member_topicdetail(self):
        response = self.client.get(reverse(
            'qanda.views.topicdetail',
            kwargs={'topic_slug': 'staff-and-members-topic'}
            ))

        self.assertEqual(response.status_code, 200)
        got_questions = response.context['question_list']
        got_slugs = [q.slug for q in got_questions]
        try:
            self.assertItemsEqual(MEMBERS_QUESTION_SLUGS, got_slugs)
        except AttributeError:
            self.assertEqual(sorted(MEMBERS_QUESTION_SLUGS), sorted(got_slugs))

    def test_member_topicdetail_noaccess(self):
        response = self.client.get(reverse(
            'qanda.views.topicdetail',
            kwargs={'topic_slug': 'staff-topic'}
            ))
        self.assertEqual(response.status_code, 404)

    def test_member_questiondetail(self):
        response = self.client.get(reverse(
            'qanda.views.questiondetail',
            kwargs={
                'topic_slug': MEMBERS_TOPIC_SLUG,
                'question_slug': MEMBERS_QUESTION_SLUG
                }
            ))
        self.assertEqual(response.status_code, 200)
        obj = response.context['object']
        self.assertEqual(obj.slug, MEMBERS_QUESTION_SLUG)
        self.assertEqual(obj.topic.slug, MEMBERS_TOPIC_SLUG)

    def test_member_questiondetail_noaccess(self):
        response = self.client.get(reverse(
            'qanda.views.questiondetail',
            kwargs={
                'topic_slug': STAFF_TOPIC_SLUG,
                'question_slug': STAFF_QUESTION_SLUG
                }
            ))
        self.assertEqual(response.status_code, 404)


class AnonymousUserTestCase(TestCase):

    fixtures = ['qanda_test.json']

    def setUp(self):
        self.client = Client()

    def test_anonymous_topiclist(self):
        response = self.client.get(reverse('qanda.views.topiclist'))
        self.assertFalse(response.context['user'].is_authenticated())

        self.assertEqual(response.status_code, 200)
        got_topics = response.context['object_list']
        got_slugs = [t.slug for t in got_topics]
        try:
            self.assertItemsEqual(PUBLIC_TOPIC_SLUGS, got_slugs)
        except AttributeError:
            self.assertEqual(sorted(PUBLIC_TOPIC_SLUGS), sorted(got_slugs))

    def test_anonymous_topicdetail(self):
        response = self.client.get(reverse(
            'qanda.views.topicdetail',
            kwargs={'topic_slug': 'public-topic'}
            ))
        self.assertEqual(response.status_code, 200)
        got_questions = response.context['question_list']
        got_slugs = [q.slug for q in got_questions]
        try:
            self.assertItemsEqual(PUBLIC_QUESTION_SLUGS, got_slugs)
        except AttributeError:
            self.assertEqual(sorted(PUBLIC_QUESTION_SLUGS), sorted(got_slugs))

    def test_anonymous_topicdetail_noaccess(self):
        response = self.client.get(reverse(
            'qanda.views.topicdetail',
            kwargs={'topic_slug': 'staff-and-members-topic'}
            ))
        self.assertEqual(response.status_code, 404)

    def test_anonymous_questiondetail(self):
        response = self.client.get(reverse(
            'qanda.views.questiondetail',
            kwargs={
                'topic_slug': PUBLIC_TOPIC_SLUG,
                'question_slug': PUBLIC_QUESTION_SLUG
                }
            ))
        self.assertEqual(response.status_code, 200)
        obj = response.context['object']
        self.assertEqual(obj.slug, PUBLIC_QUESTION_SLUG)
        self.assertEqual(obj.topic.slug, PUBLIC_TOPIC_SLUG)

    def test_anonymous_questiondetail_noaccess(self):
        response = self.client.get(reverse(
            'qanda.views.questiondetail',
            kwargs={
                'topic_slug': STAFF_TOPIC_SLUG,
                'question_slug': STAFF_QUESTION_SLUG
                }
            ))
        self.assertEqual(response.status_code, 404)

        response = self.client.get(reverse(
            'qanda.views.questiondetail',
            kwargs={
                'topic_slug': MEMBERS_TOPIC_SLUG,
                'question_slug': MEMBERS_QUESTION_SLUG
                }
            ))
        self.assertEqual(response.status_code, 404)



from django.conf.urls import url, include
from django.contrib.sitemaps.views import sitemap
from qanda.sitemaps import QuestionSitemap

sitemaps = {
    'questions': QuestionSitemap,
    }

urlpatterns = [
    url(r'^faq/', include('qanda.urls')),
    url(r'^sitemap\.xml$',
        sitemap,
        {'sitemaps': sitemaps},
        name='django.contrib.sitemaps.views.sitemap',
        )
    ]



from django.conf.urls import url
from .views import TopicDetail, TopicList, QuestionDetail


urlpatterns = [

    url(r'^$',
        TopicList.as_view(),
        name='qanda.views.topiclist'
        ),
    url(r'^(?P<topic_slug>[\w-]+)/$',
        TopicDetail.as_view(),
        name='qanda.views.topicdetail'
        ),
    url(r'^(?P<topic_slug>[\w-]+)/(?P<question_slug>[\w-]+)/$',
        QuestionDetail.as_view(),
        name='qanda.views.questiondetail'
        ),
    ]
